;
.include "Header.inc"       ;Label und Werte Zuweisungen
.include "Hardware.inc"     ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000..
 rjmp  _reset               ;..mit der RESET-Vectortabelle

.include "IRQt13a.inc"      ;Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren
;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,RAMEND				;Den Stapelzeiger auf dass Ende..
 out  SPL,a					;..des SRAM setzen
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
clr  null					;Register null mit 0 belegen
;
;Ports Erstkonfiguration
; 
 sbi  DDRB,DDB0				;NPN-Basis auf Low
;
;Analog,- Digital-Wandler initialisieren. 
;ADC2 Digitaleingangszweig vom Analogzweig trennen, ADC2 ( PB4 ), 
;10-Bit und interne Referenz von 1,1V verwenden.
;Single Conversations Mode mit 150kHz 
 sbi  DIDR0,ADC2D			;Digitalzweig vom Analogzweig trennen

 ldi  a,1<<REFS0|1<<MUX1	;Adc2, 10-Bit, 1.1V Analog Referenzspannnung 
 out  ADMUX,a	

 ldi  a,1<<ADEN|1<<ADPS1|1<<ADPS0;ADC freigeben mit 9,6MHz/DIV8/8 = 150kHz
 out  ADCSRA,a
;
;Timer/Counter0: Fast PWM Mode ( 3 ), ohne Vorteiler initialisieren,
;NON-INVERT. Clear OC0A on Compare Match, set OC0A on TOP ( $FF )
;TOV0-Flag set on MAX ( $FF ).
;
 ldi  a,$01					;No Prescaler ;Pwm Freq ~ 4,69kHz	
 out  TCCR0B,a		

 ldi  a,#PwmNon				;Fast PWM; Non Invering Mode 
 out  TCCR0A,a
;
;Verschiedenes
;      			 
 ldi  a,$5b					;Internen RC-Oscillator..
 out  OSCCAL,a				;..kalibrieren

 sbi  ACSR,ACD				;Analog-Komparator abschalten.               

 ;sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
; 
_main:									    
 sbi  ADCSRA,ADSC			;ADC starten		
         
 sbis ADCSRA,ADIF			;Wandlung fertig? 	
 rjmp pc-1					;NEIN -> Wieder abfragen

 in   volt,ADCL				;Istwert..
 in	  a,ADCH				;..einlesen

 sbi  ADCSRA,ADIF			;ADC-Wandlung fertig  -> Flag loeschen

 cpi  a,$03					;If Hi Order 2 Bits = "11" then OK
 breq _error

 clr  volt					;Else Low Byte = Zero
;
;-----	
;
_error:

;* Finds Difference (Error) between the Reference Voltage in SetPoint and 
;* the Current Voltage in Volt

;* PrerrH:PrerrL = SetPoint - Volt

;* Input  Variable = #sollwert   desired Output Voltage

;* Output Variable = PrerrH:PrerrL    Signed Error.

 ldi  b,#sollwert			;Get Reference Voltage

 clr  PrerrH				;Clear Error High

 sub  b,volt				;Subtract Actual from Reference Voltage

 brcc pc+2					;Was there a Borrow?	

 dec  PrerrH				;If Yes then PrerrH = $ff; Else EOH = $00

 mov  PrerrL,b				;store in current Error
;
;----
;
_Prop:

;* AccumH:AccumL = PrerrH:PrerrL * Kp

;* Kp = 8   produced by 3 Left Shifts 

;* Input  Variable = PrerrH:PrerrL = Error and  Kp is the Proportional Gain factor

;* Output Variable = AccumH:AccumL =  Proportional Component.


 movw AccumH:AccumL,PrerrH:PrerrL	; Mov Error into Sum

 lsl  AccumL				;Multiply Sum by 8
 rol  AccumH
 lsl  AccumL				
 rol  AccumH
 lsl  AccumL				;Proportional result in Sum 
 rol  AccumH
;
;----
;
_Diff:

;* Finds difference/Delta between Present Loop Error and Previous Loop Error

;* DeltaH:DeltaL =(PaerrH:PaerrL - PrerrH:PrerrL) * Kd: Kd = 8 (Three Left Shifts)

;* Input Variables:	
;* PaerrH:PaerrL Previous Loop Error: PrerrH:PrerrL Pesent Loop Error
;* Kd = Differential Gain factor

;* Output Variable = DeltaH:DeltaL =  Differentional Component.


 movw DeltaH:DeltaL,PrerrH:PrerrL	;Move Error into Delta
	
 sub  DeltaL,PaerrL			;Subtract Previous from Current Error
 sbc  DeltaH,PaerrH			;and store in Delta

 lsl  DeltaL				;Multiply Delta by 8
 rol  DeltaH
 lsl  DeltaL				
 rol  DeltaH
 lsl  DeltaL				
 rol  DeltaH
;
;-----
;
_Integ:

;* Multiplies Present Error by Ki and adds result to IntegH:IntegL

;* IntegH:IntegL =IntegH:IntegL + PrerrH:PrerrL * Ki: Ki = 8

;* Input Variables:	
;* PrerrH:PrerrL				: Present Loop Error
;* IntegH:IntegL				: Running Error Total
;* Ki						: Integral Gain factor

;* Output Variable = IntegH:IntegL 		: Integral Component.

 movw PaerrH:PaerrL,PrerrH:PrerrL	;Update Previous Error

 lsl  PrerrL				;Multiply Error by 8
 rol  PrerrH
 lsl  PrerrL				
 rol  PrerrH
 lsl  PrerrL				
 rol  PrerrH

 add IntegL,PrerrL			;Add Result to Integral
 adc IntegH,PrerrH
;	
;-----
;
_Total:

;* Sums Proportional. Integral and Differential components

;* AccumH:AccumL =IntegH:IntegL + DeltaH:DeltaL +ProH:ProL * Kd

;* Input Variables:	IntegH:IntegL, PwmH:PwmL, DeltaH:DeltaL 

;* Output Variable:	AccumH:AccumL ;   Sum of Components.


 add  AccumL,IntegL			;Add in the Integral
 add  AccumH,IntegH

 add  AccumL,DeltaL			;Add in the Differential
 add  AccumH,DeltaH

 brpl pc+3					;Total in  Sum Negative ?
 clr  AccumH				;Yes then Clear Sum Register ...
 clr  AccumL
	
 lsr  AccumH				
 ror  AccumL
 lsr  AccumH				;Divide Sum by 8
 ror  AccumL
 lsr  AccumH
 ror  AccumL

 breq _pwm_aus				;Stellgroesse = 0?
 ldi  a,#PwmNon				;NEIN -> Fast PWM; Non Invering Mode.. 
 out  TCCR0A,a				;..einstellen

 out  OCR0A,AccumL			;Update PWM Duty Cycle

 rjmp _main
_pwm_aus:
 cbi  PWM_PORT,PWM_PORT.PWM_1;Low-Pegel am PWM-Ausgang erzeugen..
 in   a,TCCR0A				;..hierzu auf Normale Portfunktion..
 cbr  a,1<<COM0A1|1<<COM0A0	;..umschalten und jetzt wird der..
 out  TCCR0A,a				;..Portpin auf Low geschaltet.

 rjmp _main

