;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
; Headerdatei zu PID.asm
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
; Von www.loetstelle.net  PID Regler
; Smps Pid  @ 9.6Mhz 
; 
;Ueberarbeitet fuer ATtiny13A im Auslieferungszustand 
;( FUSE-BITS High = $FF ; Low = $6A ). Urspruenglich ATtiny13.
;
;Um die Theorie auszutesten und ein Gefhl fr die Materie zu erhalten, 
;entwickelte ich diesen PWM Akku Lader mit dem unten gezeigten Schaltplan.
;Ein Tiny13 mit 9.6MHz getaktet bildet das Kernstck der Anwendung.
;
;Die Ausgangsspannung (Sollgrsse) ist auf 5.25 V festgelegt und wird stndig 
;ueber einen Spannungsteilers mit der internen Referenz von1.1 Volt verglichen.
;Das Fehlersignal, entsprechend aufbereitet, wird dann dazu verwendet, die 
;Proportional / Integral und Differential Komponenten fr den Regel Algorithmus 
;zu erzeugen und damit die Pulsweite des Ausgangssignals zu regeln.
;
;Der auf Geschwindigkeit optimierte Code ist ausgiebig kommentiert und sollte 
;leicht zu modifizieren sein.
;
;
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
; Einbinden der Controllerspezifischen Definitionsdatei
;CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
;
.NOLIST                 ;List-Output ausschalten
.INCLUDE "tn13adef.inc" ;AVR-Definitionsdatei einbinden
.LIST                   ;List-Output wieder einschalten
;
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
; Arbeitsregister umbenennen
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;RRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRRR
;
;.DEF   = r0   ;
;.DEF   = r1   ;
;.DEF   = r2   ;
;.DEF   = r3   ;
.def	null		= r4  ; Zero Constant
.def	volt		= r5  ; A2D Voltage
.def	PrerrL		= r6  ; Present Error Registers
.def	PrerrH		= r7
.def	PaerrL		= r8  ; Past Error Registers	
.def	PaerrH		= r9				
.def	IntegL		= r10 ; Integral Component Registers
.def	IntegH		= r11			
.def	AccumL		= r12 ; Sum Accumulation Registers
.def	AccumH		= r13			
.def	DeltaL		= r14 ; Difference/Delta Component Registers
.def	DeltaH		= r15			
       
.def	a		= r16	  ; Scratch Registers	
.def	b		= r17	

;.DEF   = r18 ;     
;.DEF   = r19 ;
;.DEF   = r20 ;
;.DEF   = r21 ;
;.DEF   = r22 ;
;.DEF   = r23 ;
;.DEF   = r24 ;
;.DEF   = r25 ;
;.UNDEF xl  = r26 ;
;.UNDEF xh  = r27 ;
;.UNDEF yl	= r28 ;
;.UNDEF yh	= r29 ;
;.UNDEF zl             = r30   ;Verwendung als Z-Zeiger, also wie bereits in der..
;.UNDEF zh             = r31   ;..Definitionsdatei von ATMEL angegeben
;
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
; Konstanten als Symbole
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;KKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKKK
;
.equ #sollwert	= $d1			; Desired Voltage, 5.25V

.equ #PwmNon	= 1<<COM0A1|0<<COM0A0|1<<WGM01|1<<WGM00;Fast PWM, Non Inverting 

